import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.*;

/**
 * This source code provides a starting point for the development of
 * dual-platform Java2D assignments (Applications/Applets). Although the
 * elements of this configuration have been gathered from a variety of sources,
 * their organization for our ICS4U context can be attributed to some fine
 * design decisions by I. Comery (RSGC '10).
 */

public class Framework extends JApplet implements ActionListener {
	// The program's base dimensions. To correlate these dimensions with
	// Eclipse's AppleViewer
	// go to Parameters Tab of the Run Configurations...Applet to modify
	private static Dimension appDimen = new Dimension(600, 600);
	// resuable object to support menu development
	private static JMenu menu;
	// The singular panel onto which will be drawn the individual class output
	private static Content content;

	public static void main(String[] args) {
		JFrame frame = new JFrame();

		frame.setTitle("Your name's ICS4U Framework");
		// Terminate process when window is closed(Application only)
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		// Center, if desired (Application only)
		GraphicsConfiguration gc = frame.getGraphicsConfiguration();
		Rectangle rect = gc.getBounds();
		frame.setLocation((rect.width - appDimen.width) / 2,
				(rect.height - appDimen.height) / 2);

		// create alternative applet interpretation
		JApplet applet = new Framework();
		applet.init();
		frame.getContentPane().add(applet);

		// final preparations for presentation
		frame.pack();
		frame.setResizable(false);
		frame.setVisible(true);
		applet.start();
	}

	/**
	 * Entry point for browsers in launching Applet.
	 */
	public void init() {
		content = new Content();
		getContentPane().add(content);

		JMenuBar mb = new JMenuBar();
		setJMenuBar(mb);

		menu = new JMenu("Studies");
		mb.add(menu);
		addMenuItem("Permutations");
		addMenuItem("Maze");
		menu = new JMenu("About");
		mb.add(menu);
		addMenuItem("Who & What");
		JMenuItem mi = new JMenuItem();
	}

	/*
	 * This init() helper method factors out the code common to adding menu
	 * items, thereby simplifying the menu construction process.
	 */
	private void addMenuItem(String label) {
		JMenuItem mi = new JMenuItem(label);
		mi.setActionCommand(label);
		menu.add(mi);
		mi.addActionListener(this);
	}

	/**
	 * Called by browsers after the applet has finished initializing, simply
	 * creates a new image, and repaints. Called explicitly from main method for
	 * launching as an Application.
	 */
	public void start() {
		content.createNewImage();
		content.repaint();
	}

	/**
	 * Sole method required by ActionListener interface. Determines what menu
	 * item was selected by examining the ActionEvent passed by the O/S.
	 */
	public void actionPerformed(ActionEvent ev) {
		String command = ev.getActionCommand();
		// Permutations requires an animated display...
		if ("Permutations".equals(command)) {
			content.current = new Permutations();
			content.startThread(200);

		} else if ("Maze".equals(command)) {
			content.current = new Permutations();
			content.startThread(200);
			// No animation required for the Welcome screen. Simply create a new
			// BufferedImage and display it.
		} else if ("Who & What".equals(command)) {
			content.current = new About(appDimen);
			content.createNewImage();
			content.repaint();
		}

	}

	/**
	 * Container class for the many Drawable objects that lie ahead in the
	 * course. Declared private solely for the purpose of javadoc.
	 * 
	 * @author I. Comery (primary)
	 */

	public class Content extends JPanel implements Runnable {
		// The temporary buffered image used for drawing off screen...
		private BufferedImage img;
		// The reference to img's drawing canvas...
		private Graphics2D drawImg;
		// The active drawable curve...
		private Drawable current;
		// SPEED controls the delay of the utility Thread
		int sleep = 500;
		int defaultSleep = sleep;
		public Thread thread;

		/**
		 * Constructor initializes
		 * <ol>
		 * <li>Pascal's Triangle as the initial <code>Drawable</code> object</li>
		 * <li>the active canvas colors</li>
		 * <li>the off-screen BufferedImage</li>
		 * </ol>
		 */
		public Content() {
			current = new About(appDimen);
			img = new BufferedImage(appDimen.width, appDimen.height,
					BufferedImage.TYPE_INT_RGB);
			drawImg = img.createGraphics();
			setPreferredSize(appDimen);
		}

		public void startThread(int sleep) {
			thread = null;
			this.sleep = sleep;
			thread = new Thread(this);
			thread.start();
		}

		public void stopThread() {
			thread = null;

		}

		/**
		 * Creates and sets up a new <code>BufferedImage</code> with
		 * <code>this component</code>'s size, and draws the current
		 * <code>Drawable</code> onto the <code>BufferedImage</code>.
		 */
		public void createNewImage() {
			try {
				img = new BufferedImage(getWidth(), getHeight(),
						BufferedImage.TYPE_INT_RGB);
				drawImg = img.createGraphics();
				drawImg.setBackground(current.getBackgroundColor());
				drawImg.clearRect(0, 0, getWidth(), getHeight());
				current.draw(img);
			} catch (IllegalArgumentException e) {
			}
		}

		/**
		 * Returns the main <code>BufferedImage</code> being used by the
		 * <code>paintComponent</code> method.
		 * 
		 * @return the <code>BufferedImage</code> currently being used as a
		 *         graphic.
		 */
		public BufferedImage getImage() {
			return img;
		}

		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			Graphics2D g2D = (Graphics2D) g;
			g2D.drawImage(getImage(), 0, 0, this);
		}

		public void setSleep(int s) {
			sleep = s;
		}

		/**
		 * Here is the main animation loop. The loop will terminate when the
		 * respective Drawable object sets its 'completed' variable to true,
		 * thus its 'finished()' method will return true.
		 */

		public void run() {
			Thread thisThread = Thread.currentThread();
			while (thread == thisThread && !current.finished()) {
				createNewImage();
				repaint();
				try {
					Thread.sleep(sleep);
				} catch (InterruptedException e) {
				}
			}
		}

	}

	/**
	 * Initial implementation of the Drawable interface that will determine and
	 * display the number of permutations of the user-entered String. By way of
	 * example, the coonstructor populates an ArrayList of integers and the
	 * draw() method is configured to integrate itself with the looped calls
	 * from Content's run() method to yield an animated display of the integers.
	 * (But of course you'll change this to solicit and display the permutations
	 * of a user-entered String)
	 * 
	 * @author C. D'Arcy
	 */
	public class Permutations implements Drawable {
		private String original = "ICS4U";
		private final int LEFTMARGIN = 10;
		private final int TOPMARGIN = 40;
		private final int LINESPACING = 20;
		private final Color fgColor = Color.red;
		private final Color bgColor = Color.white;
		private ArrayList<Integer> perms;
		int count = 0;
		boolean completed;

		public Permutations() {
			// include code here that will acquire the original string from the
			// user
			// and populate the permutations array.
			// Here's an example....
			perms = new ArrayList<Integer>(10);
			for (int i = 0; i < 10; i++) {
				perms.add(i);
			}

		}

		/**
		 * Called by respective Drawable's createNewGraphics' method. This is
		 * where the specific drawing takes place.
		 */

		public void draw(BufferedImage img) {
			Graphics2D g = (Graphics2D) img.createGraphics();
			g.setColor(fgColor);
			int fontSize = 12;
			g.setFont(new Font("Arial", Font.BOLD, fontSize));
			g.drawString(String.format(
					"The first %d integers are...", perms.size()), LEFTMARGIN,
					15);
			// include code here that will display the permutations
			// of the original string, one per line.
			for (int i = 0; i <= count; i++) {
				g.drawString(perms.get(i).toString(), LEFTMARGIN, LINESPACING * i + TOPMARGIN);
			}
			completed = ++count == perms.size();
		}

		public Color getBackgroundColor() {
			return bgColor;
		}

		@Override
		public boolean finished() {
			// TODO Auto-generated method stub
			return completed;
		}

		@Override
		public void setCompleted() {
			// TODO Auto-generated method stub

		}

		@Override
		public void saveImage() {
			// TODO Auto-generated method stub

		}

	}

	/**
	 * Simple interface used for all Drawable classes. Every Drawable object
	 * requires a method to draw to a given <code>BufferedImage</code>, and a
	 * method returning the background color used.
	 * 
	 * @version 1.0
	 * @author iComery
	 */
	public interface Drawable {
		/**
		 * Draws itself to the parameterized <code>BufferedImage</code>.
		 * 
		 * @param drawTo
		 *            a <code>BufferedImage</code> to draw to.
		 */
		public abstract void draw(BufferedImage drawTo);

		/**
		 * Merely returns the background color for <code>this Drawable</code>.
		 * 
		 * @return the <code>Color</code> of the background for
		 *         <code>this Drawable</code>.
		 */
		public abstract Color getBackgroundColor();

		/**
		 * Sets the status of the rendering to completed. Called by Content's
		 * stopThread method to interrupt and terminate the rendering thread.
		 */
		public abstract boolean finished();

		/**
		 * Sets the status of the rendering to completed. Called by Content's
		 * stopThread method to interrupt and terminate the rendering thread.
		 */
		public abstract void setCompleted();

		public abstract void saveImage();

	}

	/**
	 * Initial implementation of the Drawable interface that will determine and
	 * display a number of rows of Pascal's Triangle. Declared private solely
	 * for the purpose of javadoc.
	 * 
	 * @author C. D'Arcy
	 * 
	 */
	public class About implements Drawable {

		private Color bgColor, fgColor;
		boolean completed;
		Dimension appDimen;
		int width, height;

		public About(Dimension appDimen) {
			this.appDimen = appDimen;
			width = appDimen.width;
			height = appDimen.height;
			bgColor = Color.white;
			fgColor = Color.black;
		}

		/**
		 * Called by respective Drawable's createNewGraphics' method. This is
		 * where the specific drawing takes place.
		 */

		public void draw(BufferedImage img) {
			Graphics2D g = (Graphics2D) img.createGraphics();
			g.setBackground(getBackgroundColor());
			g.setColor(fgColor);
			String str = "RSGC ACES";
			g.setFont(new Font("Arial", Font.BOLD, 24));
			FontMetrics fm = g.getFontMetrics();
			Rectangle2D area = fm.getStringBounds(str, g);
			g.drawString(str, (int) (width - area.getWidth()) / 2, height / 3);
			str = "Algorithmic Framework";
			g.setFont(new Font("Arial", Font.BOLD, 36));
			fm = g.getFontMetrics();
			area = fm.getStringBounds(str, g);
			g.drawString(str, (int) (width - area.getWidth()) / 2,
					height / 3 + 64);
			str = "A Catalog of Animated Renderings of Interesting Algorithms";
			g.setFont(new Font("Arial", Font.BOLD, 16));
			fm = g.getFontMetrics();
			area = fm.getStringBounds(str, g);
			g.drawString(str, (int) (width - area.getWidth()) / 2,
					height / 3 + 120);

		}

		public Color getBackgroundColor() {
			return bgColor;
		}

		@Override
		public boolean finished() {
			return true;

		}

		@Override
		public void setCompleted() {
			completed = true;

		}

		@Override
		public void saveImage() {
			// TODO Auto-generated method stub

		}

	}

}
